@@ -4,7 +4,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief common internal definitions for libyang
  *
- * Copyright (c) 2015 - 2024 CESNET, z.s.p.o.
+ * Copyright (c) 2015 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 #ifndef LY_COMMON_H_
 #define LY_COMMON_H_
 
+#include <limits.h>
 #include <pthread.h>
 #include <stddef.h>
 #include <stdint.h>
@@ -61,6 +62,11 @@ struct lysc_node;
 #define GETMACRO6(_1, _2, _3, _4, _5, _6, NAME, ...) NAME
 #define GETMACRO7(_1, _2, _3, _4, _5, _6, _7, NAME, ...) NAME
 
+/** optional in POSIX */
+#ifndef PATH_MAX
+# define PATH_MAX 4096
+#endif
+
 /******************************************************************************
  * Logger
  *****************************************************************************/
@@ -343,12 +349,14 @@ struct ly_ctx {
     struct ly_dict dict;              /**< dictionary to effectively store strings used in the context related structures */
     struct ly_set search_paths;       /**< set of directories where to search for schema's imports/includes */
     struct ly_set list;               /**< set of loaded YANG schemas */
-    ly_module_imp_clb imp_clb;        /**< optional callback for retrieving missing included or imported models */
+    ly_module_imp_clb imp_clb;        /**< optional callback for retrieving missing included or imported modules */
     void *imp_clb_data;               /**< optional private data for ::ly_ctx.imp_clb */
     struct lys_glob_unres unres;      /**< global unres, should be empty unless there are modules prepared for
                                            compilation if ::LY_CTX_EXPLICIT_COMPILE flag is set */
     uint16_t change_count;            /**< count of changes of the context, on some changes it could be incremented
                                            more times */
+    uint32_t mod_hash;                /**< hash of the current context, includes name/revision/enabled features/implement state
+                                           of every loaded module */
     uint16_t flags;                   /**< context settings, see @ref contextoptions */
 
     ly_ext_data_clb ext_clb;          /**< optional callback for providing extension-specific run-time data for extensions */
@@ -360,6 +368,13 @@ struct ly_ctx {
     struct ly_set plugins_extensions; /**< contets specific set of extension plugins */
 };
 
+/**
+ * @brief Record a change of the context, its modules.
+ *
+ * @param[in] ctx Changed context.
+ */
+void ly_ctx_new_change(struct ly_ctx *ctx);
+
 /**
  * @brief Get the (only) implemented YANG module specified by its name.
  *
@@ -671,4 +686,13 @@ LY_ERR ly_munmap(void *addr, size_t length);
  */
 LY_ERR ly_strcat(char **dest, const char *format, ...) _FORMAT_PRINTF(2, 3);
 
+/**
+ * @brief Get schema (context) for a mount point.
+ *
+ * @param[in] ext Compiled extension instance where the context is also stored.
+ * @param[out] ext_ctx Schema to use for parsing the data.
+ * @return LY_ERR value.
+ */
+LY_ERR lyplg_ext_schema_mount_get_ctx(struct lysc_ext_instance *ext, const struct ly_ctx **ext_ctx);
+
 #endif /* LY_COMMON_H_ */
