@@ -36,6 +36,9 @@
  * | 1/2/4/8 | yes | pointer to the specific integer type | little-endian integer value |
  */
 
+static LY_ERR lyplg_type_validate_int(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, const struct lyd_node *UNUSED(ctx_node), const struct lyd_node *UNUSED(tree), struct lyd_value *storage, struct ly_err_item **err);
+static LY_ERR lyplg_type_validate_uint(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, const struct lyd_node *UNUSED(ctx_node), const struct lyd_node *UNUSED(tree), struct lyd_value *storage, struct ly_err_item **err);
+
 /**
  * @brief LYB value size of each integer type.
  */
@@ -50,7 +53,6 @@ lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, con
         const struct lysc_node *UNUSED(ctx_node), struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres),
         struct ly_err_item **err)
 {
-    struct lysc_type_num *type_num = (struct lysc_type_num *)type;
     LY_ERR ret = LY_SUCCESS;
     int64_t num = 0;
     int base = 1;
@@ -155,12 +157,9 @@ lyplg_type_store_int(const struct ly_ctx *ctx, const struct lysc_type *type, con
     }
 
     if (!(options & LYPLG_TYPE_STORE_ONLY)) {
-        /* validate range of the number */
-        if (type_num->range) {
-            ret = lyplg_type_validate_range(type->basetype, type_num->range, num, storage->_canonical,
-                    strlen(storage->_canonical), err);
-            LY_CHECK_GOTO(ret, cleanup);
-        }
+        /* validate value */
+        ret = lyplg_type_validate_int(ctx, type, NULL, NULL, storage, err);
+        LY_CHECK_GOTO(ret, cleanup);
     }
 
 cleanup:
@@ -174,6 +173,48 @@ cleanup:
     return ret;
 }
 
+/**
+ * @brief Implementation of ::lyplg_type_validate_clb for the signed interger types.
+ */
+static LY_ERR
+lyplg_type_validate_int(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, const struct lyd_node *UNUSED(ctx_node),
+        const struct lyd_node *UNUSED(tree), struct lyd_value *storage, struct ly_err_item **err)
+{
+    LY_ERR ret;
+    struct lysc_type_num *type_num = (struct lysc_type_num *)type;
+    int64_t num;
+
+    LY_CHECK_ARG_RET(NULL, type, storage, err, LY_EINVAL);
+    *err = NULL;
+
+    /* set the value (matters for big-endian) and get the correct int64 number */
+    switch (type->basetype) {
+    case LY_TYPE_INT8:
+        num = storage->int8;
+        break;
+    case LY_TYPE_INT16:
+        num = storage->int16;
+        break;
+    case LY_TYPE_INT32:
+        num = storage->int32;
+        break;
+    case LY_TYPE_INT64:
+        num = storage->int64;
+        break;
+    default:
+        return LY_EINVAL;
+    }
+
+    /* validate range of the number */
+    if (type_num->range) {
+        ret = lyplg_type_validate_range(type->basetype, type_num->range, num, storage->_canonical,
+                strlen(storage->_canonical), err);
+        LY_CHECK_RET(ret);
+    }
+
+    return LY_SUCCESS;
+}
+
 LIBYANG_API_DEF LY_ERR
 lyplg_type_compare_int(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
 {
@@ -316,7 +357,6 @@ lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, co
         const struct lysc_node *UNUSED(ctx_node), struct lyd_value *storage, struct lys_glob_unres *UNUSED(unres),
         struct ly_err_item **err)
 {
-    struct lysc_type_num *type_num = (struct lysc_type_num *)type;
     LY_ERR ret = LY_SUCCESS;
     uint64_t num = 0;
     int base = 0;
@@ -401,12 +441,9 @@ lyplg_type_store_uint(const struct ly_ctx *ctx, const struct lysc_type *type, co
     }
 
     if (!(options & LYPLG_TYPE_STORE_ONLY)) {
-        /* validate range of the number */
-        if (type_num->range) {
-            ret = lyplg_type_validate_range(type->basetype, type_num->range, num, storage->_canonical,
-                    strlen(storage->_canonical), err);
-            LY_CHECK_GOTO(ret, cleanup);
-        }
+        /* validate value */
+        ret = lyplg_type_validate_uint(ctx, type, NULL, NULL, storage, err);
+        LY_CHECK_GOTO(ret, cleanup);
     }
 
 cleanup:
@@ -420,6 +457,48 @@ cleanup:
     return ret;
 }
 
+/**
+ * @brief Implementation of ::lyplg_type_validate_clb for the unsigned interger types.
+ */
+static LY_ERR
+lyplg_type_validate_uint(const struct ly_ctx *UNUSED(ctx), const struct lysc_type *type, const struct lyd_node *UNUSED(ctx_node),
+        const struct lyd_node *UNUSED(tree), struct lyd_value *storage, struct ly_err_item **err)
+{
+    LY_ERR ret;
+    struct lysc_type_num *type_num = (struct lysc_type_num *)type;
+    uint64_t num;
+
+    LY_CHECK_ARG_RET(NULL, type, storage, err, LY_EINVAL);
+    *err = NULL;
+
+    /* set the value (matters for big-endian) and get the correct int64 number */
+    switch (type->basetype) {
+    case LY_TYPE_UINT8:
+        num = storage->uint8;
+        break;
+    case LY_TYPE_UINT16:
+        num = storage->uint16;
+        break;
+    case LY_TYPE_UINT32:
+        num = storage->uint32;
+        break;
+    case LY_TYPE_UINT64:
+        num = storage->uint64;
+        break;
+    default:
+        return LY_EINVAL;
+    }
+
+    /* validate range of the number */
+    if (type_num->range) {
+        ret = lyplg_type_validate_range(type->basetype, type_num->range, num, storage->_canonical,
+                strlen(storage->_canonical), err);
+        LY_CHECK_RET(ret);
+    }
+
+    return LY_SUCCESS;
+}
+
 LIBYANG_API_DEF LY_ERR
 lyplg_type_compare_uint(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *val1, const struct lyd_value *val2)
 {
@@ -567,7 +646,7 @@ const struct lyplg_type_record plugins_integer[] = {
 
         .plugin.id = "libyang 2 - integers, version 1",
         .plugin.store = lyplg_type_store_uint,
-        .plugin.validate = NULL,
+        .plugin.validate = lyplg_type_validate_uint,
         .plugin.compare = lyplg_type_compare_uint,
         .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
@@ -581,7 +660,7 @@ const struct lyplg_type_record plugins_integer[] = {
 
         .plugin.id = "libyang 2 - integers, version 1",
         .plugin.store = lyplg_type_store_uint,
-        .plugin.validate = NULL,
+        .plugin.validate = lyplg_type_validate_uint,
         .plugin.compare = lyplg_type_compare_uint,
         .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
@@ -595,7 +674,7 @@ const struct lyplg_type_record plugins_integer[] = {
 
         .plugin.id = "libyang 2 - integers, version 1",
         .plugin.store = lyplg_type_store_uint,
-        .plugin.validate = NULL,
+        .plugin.validate = lyplg_type_validate_uint,
         .plugin.compare = lyplg_type_compare_uint,
         .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
@@ -609,7 +688,7 @@ const struct lyplg_type_record plugins_integer[] = {
 
         .plugin.id = "libyang 2 - integers, version 1",
         .plugin.store = lyplg_type_store_uint,
-        .plugin.validate = NULL,
+        .plugin.validate = lyplg_type_validate_uint,
         .plugin.compare = lyplg_type_compare_uint,
         .plugin.sort = lyplg_type_sort_uint,
         .plugin.print = lyplg_type_print_uint,
@@ -623,7 +702,7 @@ const struct lyplg_type_record plugins_integer[] = {
 
         .plugin.id = "libyang 2 - integers, version 1",
         .plugin.store = lyplg_type_store_int,
-        .plugin.validate = NULL,
+        .plugin.validate = lyplg_type_validate_int,
         .plugin.compare = lyplg_type_compare_int,
         .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
@@ -637,7 +716,7 @@ const struct lyplg_type_record plugins_integer[] = {
 
         .plugin.id = "libyang 2 - integers, version 1",
         .plugin.store = lyplg_type_store_int,
-        .plugin.validate = NULL,
+        .plugin.validate = lyplg_type_validate_int,
         .plugin.compare = lyplg_type_compare_int,
         .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
@@ -651,7 +730,7 @@ const struct lyplg_type_record plugins_integer[] = {
 
         .plugin.id = "libyang 2 - integers, version 1",
         .plugin.store = lyplg_type_store_int,
-        .plugin.validate = NULL,
+        .plugin.validate = lyplg_type_validate_int,
         .plugin.compare = lyplg_type_compare_int,
         .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
@@ -665,7 +744,7 @@ const struct lyplg_type_record plugins_integer[] = {
 
         .plugin.id = "libyang 2 - integers, version 1",
         .plugin.store = lyplg_type_store_int,
-        .plugin.validate = NULL,
+        .plugin.validate = lyplg_type_validate_int,
         .plugin.compare = lyplg_type_compare_int,
         .plugin.sort = lyplg_type_sort_int,
         .plugin.print = lyplg_type_print_int,
