@@ -150,6 +150,48 @@ lyb_parse_union(const void *lyb_data, size_t lyb_data_len, uint32_t *type_idx, c
     }
 }
 
+/**
+ * @brief For leafref failures, ensure the appropriate error is propagated, not a type validation failure.
+ *
+ * RFC7950 Section 15.5 defines the appropriate error app tag of "require-instance".
+ *
+ * @param[in,out] err Error record to be updated.
+ * @param[in] type Leafref type used to extract target path.
+ * @param[in] value Value attempted to be stored.
+ * @param[in] value_len Length of @p value.
+ * @return LY_ERR value. Only possible errors are LY_SUCCESS and LY_EMEM.
+ */
+static LY_ERR
+union_update_lref_err(struct ly_err_item *err, const struct lysc_type *type, const void *value, size_t value_len)
+{
+    const struct lysc_type_leafref *lref;
+    char *valstr = NULL;
+    int r;
+
+    if (!err || (type->basetype != LY_TYPE_LEAFREF)) {
+        /* nothing to do */
+        return LY_SUCCESS;
+    }
+
+    lref = (const struct lysc_type_leafref *)type;
+
+    /* update error-app-tag */
+    free(err->apptag);
+    err->apptag = strdup("instance-required");
+    LY_CHECK_ERR_RET(!err->apptag, LOGMEM(NULL), LY_EMEM);
+
+    valstr = strndup((const char *)value, value_len);
+    LY_CHECK_ERR_RET(!valstr, LOGMEM(NULL), LY_EMEM);
+
+    /* update error-message */
+    free(err->msg);
+    r = asprintf(&err->msg, LY_ERRMSG_NOLREF_VAL, valstr, lyxp_get_expr(lref->path));
+    free(valstr);
+    LY_CHECK_ERR_RET(r == -1, LOGMEM(NULL), LY_EMEM);
+
+    return LY_SUCCESS;
+}
+
 /**
  * @brief Store (and validate) subvalue as a specific type.
  *
@@ -187,10 +229,13 @@ union_store_type(const struct ly_ctx *ctx, struct lysc_type_union *type_u, uint3
             /* value of another type, first store the value properly and then use its JSON value for parsing */
             rc = type_u->types[ti]->plugin->store(ctx, type_u->types[ti], value, value_len, LYPLG_TYPE_STORE_ONLY,
                     subvalue->format, subvalue->prefix_data, subvalue->hints, subvalue->ctx_node, &subvalue->value, unres, err);
-            if ((rc != LY_SUCCESS) && (rc != LY_EINCOMPLETE)) {
+            if (rc && (rc != LY_EINCOMPLETE)) {
                 /* clear any leftover/freed garbage */
                 memset(&subvalue->value, 0, sizeof subvalue->value);
-                return rc;
+
+                /* if this is a leafref, lets make sure we propagate the appropriate error, and not a type validation failure */
+                union_update_lref_err(*err, type_u->types[ti], value, value_len);
+                goto cleanup;
             }
 
             assert(subvalue->value.realtype);
@@ -219,16 +264,16 @@ union_store_type(const struct ly_ctx *ctx, struct lysc_type_union *type_u, uint3
     if (options & LYPLG_TYPE_STORE_ONLY) {
         opts |= LYPLG_TYPE_STORE_ONLY;
     }
-    if (dynamic) {
-        opts |= LYPLG_TYPE_STORE_DYNAMIC;
-    }
 
     rc = type->plugin->store(ctx, type, value, value_len, opts, format, prefix_data, subvalue->hints,
             subvalue->ctx_node, &subvalue->value, unres, err);
-    if ((rc != LY_SUCCESS) && (rc != LY_EINCOMPLETE)) {
+    if (rc && (rc != LY_EINCOMPLETE)) {
         /* clear any leftover/freed garbage */
         memset(&subvalue->value, 0, sizeof subvalue->value);
-        return rc;
+
+        /* if this is a leafref, lets make sure we propagate the appropriate error, and not a type validation failure */
+        union_update_lref_err(*err, type, value, value_len);
+        goto cleanup;
     }
 
     if (validate && (rc == LY_EINCOMPLETE)) {
@@ -237,9 +282,14 @@ union_store_type(const struct ly_ctx *ctx, struct lysc_type_union *type_u, uint3
         if (rc) {
             /* validate failed, we need to free the stored value */
             type->plugin->free(ctx, &subvalue->value);
+            goto cleanup;
         }
     }
 
+cleanup:
+    if (dynamic) {
+        free((void *)value);
+    }
     return rc;
 }
 
@@ -267,8 +317,9 @@ union_find_type(const struct ly_ctx *ctx, struct lysc_type_union *type_u, struct
     LY_ARRAY_COUNT_TYPE u;
     struct ly_err_item **errs = NULL, *e;
     uint32_t *prev_lo, temp_lo = 0;
-    char *msg = NULL;
+    char *msg = NULL, *err_app_tag = NULL;
     int msg_len = 0;
+    ly_bool use_err_app_tag = 0;
 
     *err = NULL;
 
@@ -299,6 +350,8 @@ union_find_type(const struct ly_ctx *ctx, struct lysc_type_union *type_u, struct
         }
         if (msg_len == -1) {
             LY_CHECK_ERR_GOTO(!errs, ret = LY_EMEM, cleanup);
+            /* for further actions in function msg_len is just 0 */
+            msg_len = 0;
         }
         for (u = 0; u < LY_ARRAY_COUNT(type_u->types); ++u) {
             if (!errs[u]) {
@@ -306,12 +359,27 @@ union_find_type(const struct ly_ctx *ctx, struct lysc_type_union *type_u, struct
                 continue;
             }
 
+            /* use an app-tag if all the types set it or set none */
+            if (errs[u]->apptag) {
+                if (!err_app_tag) {
+                    err_app_tag = strdup(errs[u]->apptag);
+                    LY_CHECK_ERR_GOTO(!err_app_tag, ret = LY_EMEM, cleanup);
+                    use_err_app_tag = 1;
+                } else if (strcmp(errs[u]->apptag, err_app_tag)) {
+                    use_err_app_tag = 0;
+                }
+            }
+
             msg = ly_realloc(msg, msg_len + 4 + strlen(type_u->types[u]->plugin->id) + 2 + strlen(errs[u]->msg) + 2);
             LY_CHECK_ERR_GOTO(!msg, ret = LY_EMEM, cleanup);
             msg_len += sprintf(msg + msg_len, "    %s: %s\n", type_u->types[u]->plugin->id, errs[u]->msg);
         }
 
-        ret = ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "%s", msg);
+        if (!use_err_app_tag) {
+            free(err_app_tag);
+            err_app_tag = NULL;
+        }
+        ret = ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, err_app_tag, "%s", msg);
     } else if (type_idx) {
         *type_idx = u;
     }
@@ -496,26 +564,33 @@ lyplg_type_compare_union(const struct ly_ctx *ctx, const struct lyd_value *val1,
 LIBYANG_API_DEF int
 lyplg_type_sort_union(const struct ly_ctx *ctx, const struct lyd_value *val1, const struct lyd_value *val2)
 {
-    int rc = LY_SUCCESS;
+    int rc;
     LY_ARRAY_COUNT_TYPE u;
-    struct lysc_type **types;
+    struct lysc_type **types, *type;
 
     if (val1->subvalue->value.realtype == val2->subvalue->value.realtype) {
         return val1->subvalue->value.realtype->plugin->sort(ctx, &val1->subvalue->value, &val2->subvalue->value);
     }
 
     /* compare according to the order of types */
+    rc = 0;
     types = ((struct lysc_type_union *)val1->realtype)->types;
     LY_ARRAY_FOR(types, u) {
-        if (types[u] == val1->subvalue->value.realtype) {
+        if (types[u]->basetype == LY_TYPE_LEAFREF) {
+            type = ((struct lysc_type_leafref *)types[u])->realtype;
+        } else {
+            type = types[u];
+        }
+
+        if (type == val1->subvalue->value.realtype) {
             rc = 1;
             break;
-        } else if (types[u] == val2->subvalue->value.realtype) {
+        } else if (type == val2->subvalue->value.realtype) {
             rc = -1;
             break;
         }
     }
-    assert(rc != 0);
+    assert(rc);
 
     return rc;
 }
