@@ -18,9 +18,9 @@
 #include <stdint.h>
 #include <stdlib.h>
 
-#include "common.h"
 #include "context.h"
 #include "log.h"
+#include "ly_common.h"
 #include "out.h"
 #include "out_internal.h"
 #include "parser_data.h"
@@ -455,6 +455,9 @@ json_print_metadata(struct jsonpr_ctx *pctx, const struct lyd_node *node, const
     }
 
     for (meta = node->meta; meta; meta = meta->next) {
+        if (!lyd_metadata_should_print(meta)) {
+            continue;
+        }
         PRINT_COMMA;
         ly_print_(pctx->out, "%*s\"%s:%s\":%s", INDENT, meta->annotation->module->name, meta->name, DO_FORMAT ? " " : "");
         LY_CHECK_RET(json_print_value(pctx, LYD_CTX(node), &meta->value, NULL));
@@ -464,6 +467,30 @@ json_print_metadata(struct jsonpr_ctx *pctx, const struct lyd_node *node, const
     return LY_SUCCESS;
 }
 
+/**
+ * @brief Check if a value can be printed for at least one metadata.
+ *
+ * @param[in] node Node to check.
+ * @return 1 if node has printable meta otherwise 0.
+ */
+static ly_bool
+node_has_printable_meta(const struct lyd_node *node)
+{
+    struct lyd_meta *iter;
+
+    if (!node->meta) {
+        return 0;
+    }
+
+    LY_LIST_FOR(node->meta, iter) {
+        if (lyd_metadata_should_print(iter)) {
+            return 1;
+        }
+    }
+
+    return 0;
+}
+
 /**
  * @brief Print attributes/metadata of the given @p node. Accepts both regular as well as opaq nodes.
  *
@@ -477,14 +504,15 @@ json_print_attributes(struct jsonpr_ctx *pctx, const struct lyd_node *node, ly_b
 {
     const struct lys_module *wdmod = NULL;
 
-    if (node->schema && (node->schema->nodetype != LYS_CONTAINER) && (node->flags & LYD_DEFAULT) &&
-            (pctx->options & (LYD_PRINT_WD_ALL_TAG | LYD_PRINT_WD_IMPL_TAG))) {
+    if (node->schema && (node->schema->nodetype != LYS_CONTAINER) && (((node->flags & LYD_DEFAULT) &&
+            (pctx->options & (LYD_PRINT_WD_ALL_TAG | LYD_PRINT_WD_IMPL_TAG))) ||
+            ((pctx->options & LYD_PRINT_WD_ALL_TAG) && lyd_is_default(node)))) {
         /* we have implicit OR explicit default node */
         /* get with-defaults module */
         wdmod = ly_ctx_get_module_implemented(LYD_CTX(node), "ietf-netconf-with-defaults");
     }
 
-    if (node->schema && (node->meta || wdmod)) {
+    if (node->schema && (wdmod || node_has_printable_meta(node))) {
         if (inner) {
             LY_CHECK_RET(json_print_member2(pctx, lyd_parent(node), LY_VALUE_JSON, NULL, 1));
         } else {
@@ -547,19 +575,18 @@ json_print_any_content(struct jsonpr_ctx *pctx, struct lyd_node_any *any)
     LY_ERR ret = LY_SUCCESS;
     struct lyd_node *iter;
     const struct lyd_node *prev_parent;
-    uint32_t prev_opts, temp_lo = 0;
+    uint32_t prev_opts, *prev_lo, temp_lo = 0;
 
     assert(any->schema->nodetype & LYD_NODE_ANY);
 
     if ((any->schema->nodetype == LYS_ANYDATA) && (any->value_type != LYD_ANYDATA_DATATREE)) {
         LOGINT_RET(pctx->ctx);
     }
-
     if (any->value_type == LYD_ANYDATA_LYB) {
         uint32_t parser_options = LYD_PARSE_ONLY | LYD_PARSE_OPAQ | LYD_PARSE_STRICT;
 
         /* turn logging off */
-        ly_temp_log_options(&temp_lo);
+        prev_lo = ly_temp_log_options(&temp_lo);
 
         /* try to parse it into a data tree */
         if (lyd_parse_data_mem(pctx->ctx, any->value.mem, LYD_LYB, parser_options, 0, &iter) == LY_SUCCESS) {
@@ -570,7 +597,7 @@ json_print_any_content(struct jsonpr_ctx *pctx, struct lyd_node_any *any)
         }
 
         /* turn logging on again */
-        ly_temp_log_options(NULL);
+        ly_temp_log_options(prev_lo);
     }
 
     switch (any->value_type) {
@@ -793,11 +820,12 @@ json_print_leaf_list(struct jsonpr_ctx *pctx, const struct lyd_node *node)
         LY_CHECK_RET(json_print_value(pctx, LYD_CTX(node), &((const struct lyd_node_term *)node)->value, node->schema->module));
 
         if (!pctx->first_leaflist) {
-            if ((node->flags & LYD_DEFAULT) && (pctx->options & (LYD_PRINT_WD_ALL_TAG | LYD_PRINT_WD_IMPL_TAG))) {
+            if (((node->flags & LYD_DEFAULT) && (pctx->options & (LYD_PRINT_WD_ALL_TAG | LYD_PRINT_WD_IMPL_TAG))) ||
+                    ((pctx->options & LYD_PRINT_WD_ALL_TAG) && lyd_is_default(node))) {
                 /* we have implicit OR explicit default node, get with-defaults module */
                 wdmod = ly_ctx_get_module_implemented(LYD_CTX(node), "ietf-netconf-with-defaults");
             }
-            if (node->meta || wdmod) {
+            if (wdmod || node_has_printable_meta(node)) {
                 /* we will be printing metadata for these siblings */
                 pctx->first_leaflist = node;
             }
@@ -822,7 +850,7 @@ static LY_ERR
 json_print_meta_attr_leaflist(struct jsonpr_ctx *pctx)
 {
     const struct lyd_node *prev, *node, *iter;
-    const struct lys_module *wdmod = NULL;
+    const struct lys_module *wdmod = NULL, *iter_wdmod;
     const struct lyd_node_opaq *opaq = NULL;
 
     assert(pctx->first_leaflist);
@@ -848,12 +876,17 @@ json_print_meta_attr_leaflist(struct jsonpr_ctx *pctx)
     LEVEL_INC;
     LY_LIST_FOR(node, iter) {
         PRINT_COMMA;
-        if ((iter->schema && (iter->meta || (iter->flags & LYD_DEFAULT))) || (opaq && opaq->attr)) {
+        if (iter->schema && ((iter->flags & LYD_DEFAULT) || ((pctx->options & LYD_PRINT_WD_ALL_TAG) && lyd_is_default(iter)))) {
+            iter_wdmod = wdmod;
+        } else {
+            iter_wdmod = NULL;
+        }
+        if ((iter->schema && (node_has_printable_meta(iter) || iter_wdmod)) || (opaq && opaq->attr)) {
             ly_print_(pctx->out, "%*s%s", INDENT, DO_FORMAT ? "{\n" : "{");
             LEVEL_INC;
 
             if (iter->schema) {
-                LY_CHECK_RET(json_print_metadata(pctx, iter, (iter->flags & LYD_DEFAULT) ? wdmod : NULL));
+                LY_CHECK_RET(json_print_metadata(pctx, iter, iter_wdmod));
             } else {
                 LY_CHECK_RET(json_print_attribute(pctx, (struct lyd_node_opaq *)iter));
             }
@@ -861,7 +894,7 @@ json_print_meta_attr_leaflist(struct jsonpr_ctx *pctx)
             LEVEL_DEC;
             ly_print_(pctx->out, "%s%*s}", DO_FORMAT ? "\n" : "", INDENT);
         } else {
-            ly_print_(pctx->out, "null");
+            ly_print_(pctx->out, "%*snull", INDENT);
         }
         LEVEL_PRINTED;
         if (!matching_node(iter, iter->next)) {
@@ -886,8 +919,16 @@ static LY_ERR
 json_print_opaq(struct jsonpr_ctx *pctx, const struct lyd_node_opaq *node)
 {
     ly_bool first = 1, last = 1;
+    uint32_t hints;
 
-    if (node->hints & (LYD_NODEHINT_LIST | LYD_NODEHINT_LEAFLIST)) {
+    if (node->hints == LYD_HINT_DATA) {
+        /* useless and confusing hints */
+        hints = 0;
+    } else {
+        hints = node->hints;
+    }
+
+    if (hints & (LYD_NODEHINT_LIST | LYD_NODEHINT_LEAFLIST)) {
         if (node->prev->next && matching_node(node->prev, &node->node)) {
             first = 0;
         }
@@ -899,22 +940,22 @@ json_print_opaq(struct jsonpr_ctx *pctx, const struct lyd_node_opaq *node)
     if (first) {
         LY_CHECK_RET(json_print_member2(pctx, pctx->parent, node->format, &node->name, 0));
 
-        if (node->hints & (LYD_NODEHINT_LIST | LYD_NODEHINT_LEAFLIST)) {
+        if (hints & (LYD_NODEHINT_LIST | LYD_NODEHINT_LEAFLIST)) {
             LY_CHECK_RET(json_print_array_open(pctx, &node->node));
         }
-        if (node->hints & LYD_NODEHINT_LEAFLIST) {
+        if (hints & LYD_NODEHINT_LEAFLIST) {
             ly_print_(pctx->out, "%*s", INDENT);
         }
-    } else if (node->hints & LYD_NODEHINT_LEAFLIST) {
+    } else if (hints & LYD_NODEHINT_LEAFLIST) {
         ly_print_(pctx->out, ",%s%*s", DO_FORMAT ? "\n" : "", INDENT);
     }
-    if (node->child || (node->hints & LYD_NODEHINT_LIST)) {
+    if (node->child || (hints & LYD_NODEHINT_LIST) || (hints & LYD_NODEHINT_CONTAINER)) {
         LY_CHECK_RET(json_print_inner(pctx, &node->node));
         LEVEL_PRINTED;
     } else {
-        if (node->hints & LYD_VALHINT_EMPTY) {
+        if (hints & LYD_VALHINT_EMPTY) {
             ly_print_(pctx->out, "[null]");
-        } else if ((node->hints & (LYD_VALHINT_BOOLEAN | LYD_VALHINT_DECNUM)) && !(node->hints & LYD_VALHINT_NUM64)) {
+        } else if ((hints & (LYD_VALHINT_BOOLEAN | LYD_VALHINT_DECNUM)) && !(hints & LYD_VALHINT_NUM64)) {
             ly_print_(pctx->out, "%s", node->value);
         } else {
             /* string or a large number */
@@ -922,7 +963,7 @@ json_print_opaq(struct jsonpr_ctx *pctx, const struct lyd_node_opaq *node)
         }
         LEVEL_PRINTED;
 
-        if (!(node->hints & LYD_NODEHINT_LEAFLIST)) {
+        if (!(hints & LYD_NODEHINT_LEAFLIST)) {
             /* attributes */
             json_print_attributes(pctx, (const struct lyd_node *)node, 0);
         } else if (!pctx->first_leaflist && node->attr) {
@@ -931,7 +972,7 @@ json_print_opaq(struct jsonpr_ctx *pctx, const struct lyd_node_opaq *node)
         }
 
     }
-    if (last && (node->hints & (LYD_NODEHINT_LIST | LYD_NODEHINT_LEAFLIST))) {
+    if (last && (hints & (LYD_NODEHINT_LIST | LYD_NODEHINT_LEAFLIST))) {
         json_print_array_close(pctx);
         LEVEL_PRINTED;
     }
