@@ -3,7 +3,7 @@
  * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief LYB printer for libyang data structure
  *
- * Copyright (c) 2020 - 2022 CESNET, z.s.p.o.
+ * Copyright (c) 2020 - 2025 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -70,6 +70,18 @@ lyb_ptr_equal_cb(void *val1_p, void *val2_p, ly_bool UNUSED(mod), void *UNUSED(c
     return 0;
 }
 
+/**
+ * @brief Hash table equal callback for checking collisions.
+ *
+ * Implementation of ::lyht_value_equal_cb.
+ */
+static ly_bool
+lyb_col_equal_cb(void *val1_p, void *val2_p, ly_bool mod, void *cb_data)
+{
+    /* for first value check use lyb_ptr_equal_cb, for collisions lyb_hash_equal_cb */
+    return mod ? lyb_ptr_equal_cb(val1_p, val2_p, mod, cb_data) : lyb_hash_equal_cb(val1_p, val2_p, mod, cb_data);
+}
+
 /**
  * @brief Check that sibling collision hash is safe to insert into hash table.
  *
@@ -91,7 +103,6 @@ lyb_hash_sequence_check(struct ly_ht *ht, struct lysc_node *sibling, LYB_HASH ht
         return LY_SUCCESS;
     }
 
-    lyht_set_cb(ht, lyb_ptr_equal_cb);
     do {
         int64_t j;
 
@@ -103,15 +114,13 @@ lyb_hash_sequence_check(struct ly_ht *ht, struct lysc_node *sibling, LYB_HASH ht
         }
         if (j == -1) {
             /* all whole hash sequences of nodes inserted with last hash col ID compare_col_id collide */
-            lyht_set_cb(ht, lyb_hash_equal_cb);
             return LY_EEXIST;
         }
 
         /* get next node inserted with last hash col ID ht_col_id */
-    } while (!lyht_find_next_with_collision_cb(ht, col_node, lyb_get_hash(*col_node, ht_col_id), lyb_hash_equal_cb,
+    } while (!lyht_find_next_with_collision_cb(ht, col_node, lyb_get_hash(*col_node, ht_col_id), lyb_col_equal_cb,
             (void **)&col_node));
 
-    lyht_set_cb(ht, lyb_hash_equal_cb);
     return LY_SUCCESS;
 }
 
@@ -453,25 +462,21 @@ lyb_write_string(const char *str, size_t str_len, uint8_t len_size, struct ly_ou
  *
  * @param[in] out Out structure.
  * @param[in] mod Module to print.
- * @param[in] with_features Whether to also print enabled features or not.
  * @param[in] lybctx LYB context.
  * @return LY_ERR value.
  */
 static LY_ERR
-lyb_print_model(struct ly_out *out, const struct lys_module *mod, ly_bool with_features, struct lylyb_ctx *lybctx)
+lyb_print_module(struct ly_out *out, const struct lys_module *mod, struct lylyb_ctx *lybctx)
 {
     LY_ERR rc = LY_SUCCESS;
     uint16_t revision;
-    struct ly_set feat_set = {0};
-    struct lysp_feature *f = NULL;
-    uint32_t i = 0;
     int r;
 
-    /* model name length and model name */
+    /* module name length and module name */
     LY_CHECK_GOTO(rc = lyb_write_string(mod->name, 0, sizeof(uint16_t), out, lybctx), cleanup);
 
-    /* model revision as XXXX XXXX XXXX XXXX (2B) (year is offset from 2000)
-     *                   YYYY YYYM MMMD DDDD */
+    /* module revision as XXXX XXXX XXXX XXXX (2B) (year is offset from 2000)
+     *                    YYYY YYYM MMMD DDDD */
     revision = 0;
     if (mod->revision) {
         r = atoi(mod->revision);
@@ -491,89 +496,13 @@ lyb_print_model(struct ly_out *out, const struct lys_module *mod, ly_bool with_f
     }
     LY_CHECK_GOTO(rc = lyb_write_number(revision, sizeof revision, out, lybctx), cleanup);
 
-    if (with_features) {
-        /* collect enabled module features */
-        while ((f = lysp_feature_next(f, mod->parsed, &i))) {
-            if (f->flags & LYS_FENABLED) {
-                LY_CHECK_GOTO(rc = ly_set_add(&feat_set, f, 1, NULL), cleanup);
-            }
-        }
-
-        /* print enabled feature count and their names */
-        LY_CHECK_GOTO(rc = lyb_write_number(feat_set.count, sizeof(uint16_t), out, lybctx), cleanup);
-        for (i = 0; i < feat_set.count; ++i) {
-            f = feat_set.objs[i];
-            LY_CHECK_GOTO(rc = lyb_write_string(f->name, 0, sizeof(uint16_t), out, lybctx), cleanup);
-        }
-    }
-
     /* fill cached hashes, if not already */
     lyb_cache_module_hash(mod);
 
 cleanup:
-    ly_set_erase(&feat_set, NULL);
     return rc;
 }
 
-/**
- * @brief Print all used YANG modules.
- *
- * @param[in] out Out structure.
- * @param[in] root Data root.
- * @param[in] lybctx LYB context.
- * @return LY_ERR value.
- */
-static LY_ERR
-lyb_print_data_models(struct ly_out *out, const struct lyd_node *root, struct lylyb_ctx *lybctx)
-{
-    struct ly_set *set;
-    LY_ARRAY_COUNT_TYPE u;
-    LY_ERR ret = LY_SUCCESS;
-    struct lys_module *mod;
-    const struct lyd_node *elem, *node;
-    uint32_t i;
-
-    LY_CHECK_RET(ly_set_new(&set));
-
-    /* collect all data node modules */
-    LY_LIST_FOR(root, elem) {
-        LYD_TREE_DFS_BEGIN(elem, node) {
-            if (node->schema) {
-                mod = node->schema->module;
-                ret = ly_set_add(set, mod, 0, NULL);
-                LY_CHECK_GOTO(ret, cleanup);
-
-                /* add also their modules deviating or augmenting them */
-                LY_ARRAY_FOR(mod->deviated_by, u) {
-                    ret = ly_set_add(set, mod->deviated_by[u], 0, NULL);
-                    LY_CHECK_GOTO(ret, cleanup);
-                }
-                LY_ARRAY_FOR(mod->augmented_by, u) {
-                    ret = ly_set_add(set, mod->augmented_by[u], 0, NULL);
-                    LY_CHECK_GOTO(ret, cleanup);
-                }
-
-                /* only top-level nodes are processed */
-                LYD_TREE_DFS_continue = 1;
-            }
-
-            LYD_TREE_DFS_END(elem, node);
-        }
-    }
-
-    /* now write module count on 2 bytes */
-    LY_CHECK_GOTO(ret = lyb_write_number(set->count, 2, out, lybctx), cleanup);
-
-    /* and all the used models */
-    for (i = 0; i < set->count; ++i) {
-        LY_CHECK_GOTO(ret = lyb_print_model(out, set->objs[i], 1, lybctx), cleanup);
-    }
-
-cleanup:
-    ly_set_free(set, NULL);
-    return ret;
-}
-
 /**
  * @brief Print LYB magic number.
  *
@@ -598,14 +527,24 @@ lyb_print_magic_number(struct ly_out *out)
  * @return LY_ERR value.
  */
 static LY_ERR
-lyb_print_header(struct ly_out *out)
+lyb_print_header(struct ly_out *out, const struct ly_ctx *ctx)
 {
     uint8_t byte = 0;
+    uint32_t hash;
 
-    /* version, future flags */
-    byte |= LYB_VERSION_NUM;
+    /* version, hash algorithm (flags) */
+    byte |= LYB_HEADER_VERSION_NUM;
+    byte |= LYB_HEADER_HASH_ALG;
 
-    LY_CHECK_RET(ly_write_(out, (char *)&byte, 1));
+    LY_CHECK_RET(ly_write_(out, (char *)&byte, sizeof byte));
+
+    /* context hash, if not printing empty data */
+    if (ctx) {
+        hash = ly_ctx_get_modules_hash(ctx);
+    } else {
+        hash = 0;
+    }
+    LY_CHECK_RET(ly_write_(out, (char *)&hash, sizeof hash));
 
     return LY_SUCCESS;
 }
@@ -780,7 +719,7 @@ lyb_print_metadata(struct ly_out *out, const struct lyd_node *node, struct lyd_l
 
     if (wd_mod) {
         /* write the "default" metadata */
-        LY_CHECK_RET(lyb_print_model(out, wd_mod, 0, lybctx->lybctx));
+        LY_CHECK_RET(lyb_print_module(out, wd_mod, lybctx->lybctx));
         LY_CHECK_RET(lyb_write_string("default", 0, sizeof(uint16_t), out, lybctx->lybctx));
         LY_CHECK_RET(lyb_write_string("true", 0, sizeof(uint16_t), out, lybctx->lybctx));
     }
@@ -791,8 +730,8 @@ lyb_print_metadata(struct ly_out *out, const struct lyd_node *node, struct lyd_l
             continue;
         }
 
-        /* model */
-        LY_CHECK_RET(lyb_print_model(out, iter->annotation->module, 0, lybctx->lybctx));
+        /* module */
+        LY_CHECK_RET(lyb_print_module(out, iter->annotation->module, lybctx->lybctx));
 
         /* annotation name with length */
         LY_CHECK_RET(lyb_write_string(iter->name, 0, sizeof(uint16_t), out, lybctx->lybctx));
@@ -1220,9 +1159,9 @@ lyb_print_node(struct ly_out *out, const struct lyd_node **printed_node, struct
     /* write node type */
     LY_CHECK_RET(lyb_print_lyb_type(out, node, lybctx));
 
-    /* write model info first */
+    /* write module info first */
     if (node->schema && ((node->flags & LYD_EXT) || !lysc_data_parent(node->schema))) {
-        LY_CHECK_RET(lyb_print_model(out, node->schema->module, 0, lybctx->lybctx));
+        LY_CHECK_RET(lyb_print_module(out, node->schema->module, lybctx->lybctx));
     }
 
     if (node->flags & LYD_EXT) {
@@ -1304,6 +1243,7 @@ lyb_print_data(struct ly_out *out, const struct lyd_node *root, uint32_t options
     lybctx->print_options = options;
     if (root) {
         lybctx->lybctx->ctx = ctx;
+        assert(ctx->mod_hash);
 
         if (root->schema && lysc_data_parent(root->schema)) {
             LOGERR(lybctx->lybctx->ctx, LY_EINVAL, "LYB printer supports only printing top-level nodes.");
@@ -1316,13 +1256,10 @@ lyb_print_data(struct ly_out *out, const struct lyd_node *root, uint32_t options
     LY_CHECK_GOTO(ret = lyb_print_magic_number(out), cleanup);
 
     /* LYB header */
-    LY_CHECK_GOTO(ret = lyb_print_header(out), cleanup);
+    LY_CHECK_GOTO(ret = lyb_print_header(out, lybctx->lybctx->ctx), cleanup);
 
-    /* all used models */
-    LY_CHECK_GOTO(ret = lyb_print_data_models(out, root, lybctx->lybctx), cleanup);
-
-    ret = lyb_print_siblings(out, root, lybctx);
-    LY_CHECK_GOTO(ret, cleanup);
+    /* all the top-level siblings, recursively */
+    LY_CHECK_GOTO(ret = lyb_print_siblings(out, root, lybctx), cleanup);
 
     /* ending zero byte */
     LY_CHECK_GOTO(ret = lyb_write(out, &zero, sizeof zero, lybctx->lybctx), cleanup);
