@@ -696,17 +696,16 @@ cleanup:
  * @param[in] revision Optional revision of the (sub)module to load, if NULL the newest revision is being loaded.
  * @param[in] main_ctx Parser context of the main module in case of loading submodule.
  * @param[in] main_name Main module name in case of loading submodule.
- * @param[in] required Module is required so error (even if the input file not found) are important. If 0, there is some
- * backup and it is actually ok if the input data are not found. However, parser reports errors even in this case.
  * @param[in,out] new_mods Set of all the new mods added to the context. Includes this module and all of its imports.
- * @param[out] result Parsed YANG schema tree of the requested module (struct lys_module*) or submodule (struct lysp_submodule*).
- * If it is a module, it is already in the context!
+ * @param[out] found Whether any (sub)module was found.
+ * @param[out] result Parsed YANG schema tree of the requested module (struct lys_module *) or submodule
+ * (struct lysp_submodule *). If a module, it has already been inserted into the context.
  * @return LY_SUCCESS on success.
  * @return LY_ERR on error.
  */
 static LY_ERR
 lys_parse_localfile(struct ly_ctx *ctx, const char *name, const char *revision, struct lysp_ctx *main_ctx,
-        const char *main_name, ly_bool required, struct ly_set *new_mods, void **result)
+        const char *main_name, struct ly_set *new_mods, ly_bool *found, void **result)
 {
     struct ly_in *in;
     char *filepath = NULL;
@@ -715,14 +714,13 @@ lys_parse_localfile(struct ly_ctx *ctx, const char *name, const char *revision,
     LY_ERR ret = LY_SUCCESS;
     struct lysp_load_module_data mod_data = {0};
 
+    *found = 0;
+    *result = NULL;
+
     LY_CHECK_RET(lys_search_localfile(ly_ctx_get_searchdirs(ctx), !(ctx->flags & LY_CTX_DISABLE_SEARCHDIR_CWD), name,
             revision, &filepath, &format));
     if (!filepath) {
-        if (required) {
-            LOGERR(ctx, LY_ENOTFOUND, "Data model \"%s%s%s\" not found in local searchdirs.", name, revision ? "@" : "",
-                    revision ? revision : "");
-            ret = LY_ENOTFOUND;
-        }
+        LOGVRB("Module \"%s%s%s\" not found in local searchdirs.", name, revision ? "@" : "", revision ? revision : "");
         goto cleanup;
     }
 
@@ -736,7 +734,7 @@ lys_parse_localfile(struct ly_ctx *ctx, const char *name, const char *revision,
     mod_data.path = filepath;
     mod_data.submoduleof = main_name;
     if (main_ctx) {
-        ret = lys_parse_submodule(ctx, in, format, main_ctx, &mod_data, new_mods, (struct lysp_submodule **)&mod);
+        ret = lys_parse_submodule(ctx, in, format, main_ctx, &mod_data, 1, new_mods, (struct lysp_submodule **)&mod);
     } else {
         ret = lys_parse_in(ctx, in, format, &mod_data, new_mods, (struct lys_module **)&mod);
 
@@ -744,6 +742,7 @@ lys_parse_localfile(struct ly_ctx *ctx, const char *name, const char *revision,
     ly_in_free(in, 1);
     LY_CHECK_GOTO(ret, cleanup);
 
+    *found = 1;
     *result = mod;
 
 cleanup:
@@ -764,12 +763,13 @@ cleanup:
  * @return LY_ERR on error.
  */
 static LY_ERR
-lys_parse_load_from_clb_or_file(struct ly_ctx *ctx, const char *name, const char *revision,
+lys_load_mod_from_clb_or_file(struct ly_ctx *ctx, const char *name, const char *revision,
         struct lys_module *mod_latest, struct ly_set *new_mods, struct lys_module **mod)
 {
     LY_ERR r;
     const char *module_data = NULL;
     LYS_INFORMAT format = LYS_IN_UNKNOWN;
+    ly_bool clb_used, searchdirs_used, found = 0;
 
     void (*module_data_free)(void *module_data, void *user_data) = NULL;
     struct lysp_load_module_data mod_data = {0};
@@ -777,17 +777,30 @@ lys_parse_load_from_clb_or_file(struct ly_ctx *ctx, const char *name, const char
 
     *mod = NULL;
 
-    if (mod_latest && (!ctx->imp_clb || (mod_latest->latest_revision & LYS_MOD_LATEST_IMPCLB)) &&
-            ((ctx->flags & LY_CTX_DISABLE_SEARCHDIRS) || (mod_latest->latest_revision & LYS_MOD_LATEST_SEARCHDIRS))) {
-        /* we are not able to find a newer revision */
-        return LY_SUCCESS;
+    if (!ctx->imp_clb) {
+        /* no callback to call */
+        clb_used = 1;
+    } else if (mod_latest && (mod_latest->latest_revision & LYS_MOD_LATEST_IMPCLB)) {
+        /* already have the latest revision returned by the callback */
+        clb_used = 1;
+    } else {
+        clb_used = 0;
     }
 
-    if (!(ctx->flags & LY_CTX_PREFER_SEARCHDIRS)) {
-search_clb:
-        /* check there is a callback and should be called */
-        if (ctx->imp_clb && (!mod_latest || !(mod_latest->latest_revision & LYS_MOD_LATEST_IMPCLB))) {
+    if (ctx->flags & LY_CTX_DISABLE_SEARCHDIRS) {
+        /* searchdirs disabled */
+        searchdirs_used = 1;
+    } else if (mod_latest && (mod_latest->latest_revision & LYS_MOD_LATEST_SEARCHDIRS)) {
+        /* already have the latest revision found in the searchdirs */
+        searchdirs_used = 1;
+    } else {
+        searchdirs_used = 0;
+    }
+
+    while (!found && (!clb_used || !searchdirs_used)) {
+        if ((!(ctx->flags & LY_CTX_PREFER_SEARCHDIRS) || searchdirs_used) && !clb_used) {
             if (!ctx->imp_clb(name, revision, NULL, NULL, ctx->imp_clb_data, &format, &module_data, &module_data_free)) {
+                /* parse the module returned by the callback */
                 LY_CHECK_RET(ly_in_new_memory(module_data, &in));
                 mod_data.name = name;
                 mod_data.revision = revision;
@@ -797,26 +810,23 @@ search_clb:
                     module_data_free((void *)module_data, ctx->imp_clb_data);
                 }
                 LY_CHECK_RET(r);
+
+                found = 1;
+            }
+            clb_used = 1;
+
+            if (*mod && !revision) {
+                /* we got the latest revision module from the callback */
+                (*mod)->latest_revision |= LYS_MOD_LATEST_IMPCLB;
+            }
+        } else if (!searchdirs_used) {
+            LY_CHECK_RET(lys_parse_localfile(ctx, name, revision, NULL, NULL, new_mods, &found, (void **)mod));
+            searchdirs_used = 1;
+
+            if (*mod && !revision) {
+                /* we got the latest revision module in the searchdirs */
+                (*mod)->latest_revision |= LYS_MOD_LATEST_SEARCHDIRS;
             }
-        }
-        if (*mod && !revision) {
-            /* we got the latest revision module from the callback */
-            (*mod)->latest_revision |= LYS_MOD_LATEST_IMPCLB;
-        } else if (!*mod && !(ctx->flags & LY_CTX_PREFER_SEARCHDIRS)) {
-            goto search_file;
-        }
-    } else {
-search_file:
-        /* check we can use searchdirs and that we should */
-        if (!(ctx->flags & LY_CTX_DISABLE_SEARCHDIRS) &&
-                (!mod_latest || !(mod_latest->latest_revision & LYS_MOD_LATEST_SEARCHDIRS))) {
-            LY_CHECK_RET(lys_parse_localfile(ctx, name, revision, NULL, NULL, mod_latest ? 0 : 1, new_mods, (void **)mod));
-        }
-        if (*mod && !revision) {
-            /* we got the latest revision module in the searchdirs */
-            (*mod)->latest_revision |= LYS_MOD_LATEST_IMPCLB;
-        } else if (!*mod && (ctx->flags & LY_CTX_PREFER_SEARCHDIRS)) {
-            goto search_clb;
         }
     }
 
@@ -918,7 +928,7 @@ lys_parse_load(struct ly_ctx *ctx, const char *name, const char *revision, struc
 
     if (!*mod) {
         /* No suitable module in the context, try to load it. */
-        LY_CHECK_RET(lys_parse_load_from_clb_or_file(ctx, name, revision, mod_latest, new_mods, mod));
+        LY_CHECK_RET(lys_load_mod_from_clb_or_file(ctx, name, revision, mod_latest, new_mods, mod));
 
         /* Update the latest_revision flag - here we have selected the latest available schema,
          * consider that even the callback provides correct latest revision.
@@ -1111,11 +1121,93 @@ lysp_inject_submodule(struct lysp_ctx *pctx, struct lysp_include *inc)
     return LY_SUCCESS;
 }
 
+/**
+ * @brief Load submodule from searchdirs or from callback.
+ *
+ * Parsing another (YANG 1.0) submodule can cause inserting submodule's include into the main module moving
+ * existing includes (changing any pointers to includes).
+ *
+ * @param[in] pctx Parser context to use.
+ * @param[in] name Name of submodule to load.
+ * @param[in] revision Revision of submodule to load.
+ * @param[in] submod_latest Submodule with the latest revision found in context, otherwise set to NULL.
+ * @param[in,out] new_mods Set of all the new mods added to the context. Includes this submodule and all of its imports.
+ * @param[out] submod Parsed submodule, may be NULL if an adequate submodule has already been found before.
+ * @return LY_SUCCESS on success.
+ * @return LY_ERR on error.
+ */
+static LY_ERR
+lysp_load_submod_from_clb_or_file(struct lysp_ctx *pctx, const char *name, const char *revision,
+        struct lysp_submodule *submod_latest, struct ly_set *new_mods, struct lysp_submodule **submod)
+{
+    LY_ERR r;
+    struct ly_ctx *ctx = PARSER_CTX(pctx);
+    ly_bool clb_used, searchdirs_used, found = 0;
+    const char *submodule_data = NULL;
+    LYS_INFORMAT format = LYS_IN_UNKNOWN;
+
+    void (*submodule_data_free)(void *module_data, void *user_data) = NULL;
+    struct lysp_load_module_data mod_data = {0};
+    struct ly_in *in;
+
+    assert(!submod_latest || (submod_latest->latest_revision != 2));
+
+    *submod = NULL;
+
+    if (!ctx->imp_clb) {
+        /* no callback to call */
+        clb_used = 1;
+    } else {
+        clb_used = 0;
+    }
+
+    if (ctx->flags & LY_CTX_DISABLE_SEARCHDIRS) {
+        /* searchdirs disabled */
+        searchdirs_used = 1;
+    } else {
+        searchdirs_used = 0;
+    }
+
+    while (!found && (!clb_used || !searchdirs_used)) {
+        if ((!(ctx->flags & LY_CTX_PREFER_SEARCHDIRS) || searchdirs_used) && !clb_used) {
+            if (!ctx->imp_clb(PARSER_CUR_PMOD(pctx)->mod->name, NULL, name, revision, ctx->imp_clb_data, &format,
+                    &submodule_data, &submodule_data_free)) {
+                /* parse the submodule returned by the callback */
+                LY_CHECK_RET(ly_in_new_memory(submodule_data, &in));
+                mod_data.name = name;
+                mod_data.revision = revision;
+                mod_data.submoduleof = PARSER_CUR_PMOD(pctx)->mod->name;
+                r = lys_parse_submodule(ctx, in, format, pctx->main_ctx, &mod_data, 0, new_mods, submod);
+                ly_in_free(in, 0);
+                if (submodule_data_free) {
+                    submodule_data_free((void *)submodule_data, ctx->imp_clb_data);
+                }
+                LY_CHECK_RET(r);
+
+                found = 1;
+            }
+            clb_used = 1;
+        } else if (!searchdirs_used) {
+            LY_CHECK_RET(lys_parse_localfile(ctx, name, revision, pctx->main_ctx,
+                    PARSER_CUR_PMOD(pctx->main_ctx)->mod->name, new_mods, &found, (void **)submod));
+            searchdirs_used = 1;
+        }
+    }
+
+    if (!*submod && !submod_latest) {
+        LOGVAL(ctx, LYVE_REFERENCE, "Including \"%s\" submodule into \"%s\" failed, not found.", name,
+                PARSER_CUR_PMOD(pctx)->is_submod ? ((struct lysp_submodule *)PARSER_CUR_PMOD(pctx))->name :
+                PARSER_CUR_PMOD(pctx)->mod->name);
+        return LY_ENOTFOUND;
+    }
+
+    return LY_SUCCESS;
+}
+
 LY_ERR
 lysp_load_submodules(struct lysp_ctx *pctx, struct lysp_module *pmod, struct ly_set *new_mods)
 {
     LY_ERR r;
-    struct ly_ctx *ctx = PARSER_CTX(pctx);
     struct lysp_submodule *submod;
     struct lysp_include *inc;
     LY_ARRAY_COUNT_TYPE u;
@@ -1127,7 +1219,6 @@ lysp_load_submodules(struct lysp_ctx *pctx, struct lysp_module *pmod, struct ly_
             continue;
         }
 
-        submod = NULL;
         submod_included = 1;
         if (pmod->is_submod) {
             /* try to find the submodule in the main module or its submodules */
@@ -1136,79 +1227,39 @@ lysp_load_submodules(struct lysp_ctx *pctx, struct lysp_module *pmod, struct ly_
                 submod_included = 0;
             } else if (r) {
                 return r;
+            } else if (inc->submodule->latest_revision == 2) {
+                /* submodule found and is the latest existing revision */
+                continue;
             }
         }
 
         /* try to use currently parsed submodule */
         r = lysp_parsed_mods_get_submodule(pctx, inc);
-        LY_CHECK_RET(r != LY_ENOT, r);
-
-        /* submodule not present in the main module, get the input data and parse it */
-        if (!(ctx->flags & LY_CTX_PREFER_SEARCHDIRS)) {
-search_clb:
-            if (ctx->imp_clb) {
-                const char *submodule_data = NULL;
-                LYS_INFORMAT format = LYS_IN_UNKNOWN;
-
-                void (*submodule_data_free)(void *module_data, void *user_data) = NULL;
-                struct lysp_load_module_data mod_data = {0};
-                struct ly_in *in;
-
-                if (ctx->imp_clb(PARSER_CUR_PMOD(pctx)->mod->name, NULL, inc->name, inc->rev[0] ? inc->rev : NULL,
-                        ctx->imp_clb_data, &format, &submodule_data, &submodule_data_free) == LY_SUCCESS) {
-                    LY_CHECK_RET(ly_in_new_memory(submodule_data, &in));
-                    mod_data.name = inc->name;
-                    mod_data.revision = inc->rev[0] ? inc->rev : NULL;
-                    mod_data.submoduleof = PARSER_CUR_PMOD(pctx)->mod->name;
-                    r = lys_parse_submodule(ctx, in, format, pctx->main_ctx, &mod_data, new_mods, &submod);
-                    ly_in_free(in, 0);
-                    if (submodule_data_free) {
-                        submodule_data_free((void *)submodule_data, ctx->imp_clb_data);
-                    }
-                    LY_CHECK_RET(r);
-
-                    /* update inc pointer - parsing another (YANG 1.0) submodule can cause injecting
-                     * submodule's include into main module, where it is missing */
-                    inc = &pmod->includes[u];
-                }
-            }
-            if (!submod && !(ctx->flags & LY_CTX_PREFER_SEARCHDIRS)) {
-                goto search_file;
-            }
-        } else {
-search_file:
-            if (!(ctx->flags & LY_CTX_DISABLE_SEARCHDIRS)) {
-                /* submodule was not received from the callback or there is no callback set */
-                lys_parse_localfile(ctx, inc->name, inc->rev[0] ? inc->rev : NULL, pctx->main_ctx,
-                        PARSER_CUR_PMOD(pctx->main_ctx)->mod->name, 1, new_mods, (void **)&submod);
-
-                /* update inc pointer - parsing another (YANG 1.0) submodule can cause injecting
-                 * submodule's include into main module, where it is missing */
-                inc = &pmod->includes[u];
-            }
-            if (!submod && (ctx->flags & LY_CTX_PREFER_SEARCHDIRS)) {
-                goto search_clb;
-            }
+        if (r && (r != LY_ENOT)) {
+            return r;
+        } else if (!r && ((inc->submodule->latest_revision == 2) || inc->submodule->parsing)) {
+            /* use the module if we already have its latest revision or if it is being parsed, we have to stop looking
+             * recursively for latest revisions at some point to prevent infinite recursion */
+            continue;
         }
-        if (submod) {
-            if (!inc->rev[0] && (submod->latest_revision == 1)) {
-                /* update the latest_revision flag - here we have selected the latest available schema,
-                 * consider that even the callback provides correct latest revision */
-                submod->latest_revision = 2;
-            }
 
+        /* try to load the submodule */
+        LY_CHECK_RET(lysp_load_submod_from_clb_or_file(pctx, inc->name, inc->rev[0] ? inc->rev : NULL, inc->submodule,
+                new_mods, &submod));
+
+        if (submod) {
+            /* update inc pointer - parsing another (YANG 1.0) submodule can cause injecting
+             * submodule's include into main module, where it is missing */
+            inc = &pmod->includes[u];
+
+            assert(!inc->submodule);
             inc->submodule = submod;
+
             if (!submod_included) {
                 /* the submodule include is not present in YANG 1.0 main module - add it there */
-                LY_CHECK_RET(lysp_inject_submodule(pctx, &pmod->includes[u]));
+                LY_CHECK_RET(lysp_inject_submodule(pctx, inc));
             }
         }
-        if (!inc->submodule) {
-            LOGVAL(ctx, LYVE_REFERENCE, "Including \"%s\" submodule into \"%s\" failed.", inc->name,
-                    PARSER_CUR_PMOD(pctx)->is_submod ? ((struct lysp_submodule *)PARSER_CUR_PMOD(pctx))->name :
-                    PARSER_CUR_PMOD(pctx)->mod->name);
-            return LY_EVALID;
-        }
     }
 
     return LY_SUCCESS;
