@@ -539,7 +539,7 @@ struct lysp_load_module_data {
 /**
  * @brief Parse a module and add it into the context.
  *
- * @param[in] ctx libyang context where to process the data model.
+ * @param[in] ctx libyang context to use.
  * @param[in] in Input structure.
  * @param[in] format Format of the input data (YANG or YIN).
  * @param[in] mod_data Optional expected module data to check.
@@ -556,17 +556,19 @@ LY_ERR lys_parse_in(struct ly_ctx *ctx, struct ly_in *in, LYS_INFORMAT format,
  *
  * The latest_revision flag of submodule is updated.
  *
- * @param[in] ctx libyang context where to process the data model.
+ * @param[in] ctx libyang context to use.
  * @param[in] in Input structure.
  * @param[in] format Format of the input data (YANG or YIN).
  * @param[in] main_ctx Parser context of the main module.
  * @param[in] mod_data Optional expected module data to check.
+ * @param[in] in_searchdirs Whether the submodule was found in the searchdirs.
  * @param[in] new_mods Set of all the new mods added to the context. Includes this module and all of its imports.
- * @param[out] submodule Parsed submodule.
+ * @param[out] submodule Parsed submodule, may be NULL even on success if an adequate submodule has already been parsed.
  * @return LY_ERR value.
  */
 LY_ERR lys_parse_submodule(struct ly_ctx *ctx, struct ly_in *in, LYS_INFORMAT format, struct lysp_ctx *main_ctx,
-        const struct lysp_load_module_data *mod_data, struct ly_set *new_mods, struct lysp_submodule **submodule);
+        const struct lysp_load_module_data *mod_data, ly_bool in_searchdirs, struct ly_set *new_mods,
+        struct lysp_submodule **submodule);
 
 /**
  * @brief Fill filepath value if available in input handler @p in
